import { MAX_TARGETS, TARGET_WIDTH } from './Constants';
import { Subject } from './Subject';
import teddy from '../data/teddy.png';
import duck from '../data/duck.png';
import squirrel from '../data/squirrel.png';

class Target extends Subject {
  constructor(x, y, width) {
    super();
    this.x = x;
    this.y = y;
    this.width = width;
    this.height = width;
    this.visible = true;
        this.img = null;
  }

  draw() {
    if (this.visible && this.img) {
      image(this.img, this.x, this.y, this.width, this.height);
    }
  }

  getPoints() {
    return 0; // to be overridden
  }

  isHit(x, y) {
    return (
      this.visible &&
      x >= this.x &&
      x <= this.x + this.width &&
      y >= this.y &&
      y <= this.y + this.height
    );
  }

  shoot(x, y) {
    if (this.isHit(x, y)) {
      this.visible = false;
      this.notifySubscribers('target-hit', this.getPoints());
    }
  }

  update(source, ...others) {
    if (source === 'gun') {
      const [x, y] = others;
      this.shoot(x, y);
    }
  }
}

class TeddyTarget extends Target {
  constructor(x, y) {
    super(x, y, TARGET_WIDTH);
    this.img = loadImage(teddy);
  }

  getPoints() {
    return 1;
  }
}

class DuckTarget extends Target {
  constructor(x, y) {
    super(x, y, TARGET_WIDTH);
    this.img = loadImage(duck);
  }

  getPoints() {
    return 3;
  }
}

class SquirrelTarget extends Target {
  constructor(x, y) {
    super(x, y, TARGET_WIDTH);
    this.img = loadImage(squirrel);
  }

  getPoints() {
    return 5;
  }
}

class TargetFactory {
  static instance;

  static getInstance() {
    if (!TargetFactory.instance) {
      TargetFactory.instance = new TargetFactory();
    }
    return TargetFactory.instance;
  }

  getTargetsByName(targetNames, targetWidth, y) {
    const targets = [];

    for (let i = 0; i < targetNames.length && i < MAX_TARGETS; i += 1) {
      const name = targetNames[i];
      const x = 100 + i * targetWidth;

      switch (name) {
        case 'teddy':
          targets.push(new TeddyTarget(x, y));
          break;
        case 'duck':
          targets.push(new DuckTarget(x, y));
          break;
        case 'squirrel':
          targets.push(new SquirrelTarget(x, y));
          break;
      }
    }

    return targets;
  }

  getRandomTargets(numTargets, targetWidth, y) {
    const names = ['teddy', 'duck', 'squirrel'];
    const targetNames = [];

    for (let i = 0; i < numTargets; i++) {
      targetNames.push(random(names));
    }

    return this.getTargetsByName(targetNames, targetWidth, y);
  }   // create random targets
}

export {
  Target, TeddyTarget,
  DuckTarget,
  SquirrelTarget, TargetFactory
};
